# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from otcextensions.osclient.mrs.v1 import job
from otcextensions.tests.unit.osclient.mrs.v1 import fakes


class TestListJob(fakes.TestMrs):
    objects = fakes.FakeCluster.create_multiple(3)

    columns = (
        'id', 'name', 'type', 'description',
        'is_public', 'is_protected'
    )

    data = []

    for s in objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListJob, self).setUp()

        self.cmd = job.ListJob(self.app, None)

        self.client.jobs = mock.Mock()
        self.client.api_mock = self.client.jobs

    def test_default(self):
        arglist = [
        ]
        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowJob(fakes.TestMrs):
    object = fakes.FakeJob.create_one()

    columns = (
        'created_at', 'description', 'id',
        'interface', 'is_protected', 'is_public',
        'libs', 'mains', 'name', 'type', 'updated_at'
    )

    data = fakes.gen_data(object, columns)

    def setUp(self):
        super(TestShowJob, self).setUp()

        self.cmd = job.ShowJob(self.app, None)

        self.client.find_job = mock.Mock()

    def test_default(self):
        arglist = [
            'job'
        ]
        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_job.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_job.assert_called_once_with(
            'job',
            ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteJob(fakes.TestMrs):

    def setUp(self):
        super(TestDeleteJob, self).setUp()

        self.cmd = job.DeleteJob(self.app, None)

        self.client.delete_job = mock.Mock()

    def test_delete(self):
        arglist = [
            'job'
        ]
        verifylist = []
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_job.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        delete_calls = [
            mock.call(
                'job',
                ignore_missing=False),
        ]

        self.client.delete_job.assert_has_calls(delete_calls)
        self.assertEqual(1, self.client.delete_job.call_count)


class TestCreateJob(fakes.TestMrs):
    object = fakes.FakeJob.create_one()

    columns = (
        'created_at', 'description', 'id',
        'interface', 'is_protected', 'is_public',
        'libs', 'mains', 'name', 'type', 'updated_at'
    )

    data = fakes.gen_data(object, columns)

    def setUp(self):
        super(TestCreateJob, self).setUp()

        self.cmd = job.CreateJob(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client.create_job = mock.Mock()

    def test_default(self):
        arglist = [
            'test_job',
            '--type', 'MapReduce',
            '--description', 'test',
        ]
        verifylist = [
            ('name', 'test_job'),
            ('type', 'MapReduce'),
            ('description', 'test'),
        ]

        # Verify cmd is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_job.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_job.assert_called_once_with(
            type='MapReduce',
            name='test_job',
            description='test',
            is_public='false',
            is_protected='false'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestUpdateJob(fakes.TestMrs):
    object = fakes.FakeJob.create_one()

    columns = (
        'created_at', 'description', 'id',
        'interface', 'is_protected', 'is_public',
        'libs', 'mains', 'name', 'type', 'updated_at'
    )

    data = fakes.gen_data(object, columns)

    def setUp(self):
        super(TestUpdateJob, self).setUp()

        self.cmd = job.UpdateJob(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client.update_datasource = mock.Mock()

    def test_default(self):
        arglist = [
            'job_id',
            '--name', 'test_job',
            '--type', 'Hive',
            '--description', 'updated',
        ]
        verifylist = [
            ('job', 'job_id'),
            ('name', 'test_job'),
            ('type', 'Hive'),
            ('description', 'updated'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_job.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_job.assert_called_with(
            'job_id',
            ignore_missing=False)

        self.client.update_job.assert_called_once_with(
            job=mock.ANY,
            name='test_job',
            type='Hive',
            description='updated',
            is_public='false'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from otcextensions.tests.functional.osclient.nat.v2 import common


class TestGateway(common.NatTestCase):
    """Functional Tests for NAT Gateway"""

    def setUp(self):
        super(TestGateway, self).setUp()

    def test_nat_gateway_list(self):
        json_output = json.loads(self.openstack(
            'nat gateway list -f json '
        ))
        self.assertIsNotNone(json_output)

    def test_nat_gateway_list_filters(self):
        json_output = json.loads(self.openstack(
            'nat gateway list -f json '
            '--limit 1 --id 2 '
            '--name 3 --spec 4 '
            '--router-id 5 '
            '--internal-network-id 6 '
            '--project-id 7 '
            '--status active '
            '--admin-state-up True '
            '--created-at "{}"'.format(self.CURR_TIME)
        ))
        self.assertIsNotNone(json_output)

    def test_nat_gateway(self):
        nat_gateway = self.create_nat_gateway()
        nat_id = nat_gateway['id']
        nat_name = nat_gateway['name']
        router_id = nat_gateway['router_id']

        self.addCleanup(self.delete_nat_gateway)

        # List Nat Gateway By Id
        json_output = json.loads(self.openstack(
            'nat gateway list -f json'
            ' --id {}'.format(nat_id)
        ))
        self.assertEqual(json_output[0]['Name'], nat_name)
        self.assertEqual(json_output[0]['Id'], nat_id)

        # List Nat Gateway By Name
        json_output = json.loads(self.openstack(
            'nat gateway list -f json'
            ' --name {}'.format(nat_name)
        ))
        self.assertEqual(json_output[0]['Name'], nat_name)
        self.assertEqual(json_output[0]['Id'], nat_id)

        # List Nat Gateway by Router ID
        json_output = json.loads(self.openstack(
            'nat gateway list -f json'
            ' --router-id {}'.format(router_id)
        ))
        for nat_gw in json_output:
            self.assertEqual(nat_gw['Router Id'], router_id)

        # Show Nat Gateway by Name
        json_output = json.loads(self.openstack(
            'nat gateway show -f json ' + nat_name
        ))
        self.assertEqual(json_output['name'], nat_name)
        self.assertEqual(json_output['id'], nat_id)

        # Show Nat Gateway by Id
        json_output = json.loads(self.openstack(
            'nat gateway show -f json ' + nat_id
        ))
        self.assertEqual(json_output['name'], nat_name)
        self.assertEqual(json_output['id'], nat_id)

        # Update Nat Gateway
        nat_name = nat_name + "-updated"
        description = "otce cli test nat updated"
        json_output = json.loads(self.openstack(
            'nat gateway update {nat_id} '
            '--name {name} '
            '--description "{desc}" '
            '-f json'.format(
                nat_id=nat_id,
                name=nat_name,
                desc=description)
        ))
        self.assertEqual(json_output['name'], nat_name)
        self.assertEqual(json_output['description'], description)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from otcextensions.sdk.dds.v3 import instance as _instance


class Job(resource.Resource):
    base_path = '/jobs'

    allow_fetch = True

    id = resource.Body('id')
    name = resource.Body('name')
    status = resource.Body('status')
    created = resource.Body('created')
    ended = resource.Body('ended')
    progress = resource.Body('progress')
    instance = resource.Body('instance', type=_instance.Instance)

    def fetch(self, session, requires_id=True,
              base_path=None, error_message=None, **params):
        params = {
            "id": self.id,
        }
        request = self._prepare_request(
            requires_id=False,
            base_path=base_path,
            params=params
        )
        response = session.get(
            request.url,
        )
        self._translate_response(response, resource_response_key='job')
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class MicroserviceLabelSpec(resource.Resource):
    label_name = resource.Body('label_name')
    label_value = resource.Body('label_value')


class BackendServerGroup(resource.Resource):
    base_path = (f'apigw/instances/%(gateway_id)s/vpc-channels/'
                 f'%(vpc_channel_id)s/member-groups')

    allow_fetch = True
    allow_list = True
    allow_delete = True
    allow_commit = True

    gateway_id = resource.URI('gateway_id')
    vpc_channel_id = resource.URI('vpc_channel_id')
    member_group_name = resource.Body('member_group_name')
    member_group_remark = resource.Body('member_group_remark')
    member_group_weight = resource.Body('member_group_weight', type=int)
    dict_code = resource.Body('dict_code')
    microservice_version = resource.Body('microservice_version')
    microservice_port = resource.Body('microservice_port', type=int)
    microservice_labels = resource.Body('microservice_labels',
                                        type=list,
                                        list_type=MicroserviceLabelSpec)
    member_group_id = resource.Body('member_group_id')
    create_time = resource.Body('create_time')
    update_time = resource.Body('update_time')

    @staticmethod
    def create_group(session, gateway_id, vpc_channel_id, **attrs):
        uri = (f'/apigw/instances/{gateway_id}/vpc-channels/{vpc_channel_id}/'
               f'member-groups')
        response = session.post(uri, json=attrs)
        exceptions.raise_from_response(response)
        result = response.json()['member_groups']
        if isinstance(result, list):
            return [BackendServerGroup.existing(**item) for item in result]
        elif isinstance(result, dict):
            return BackendServerGroup.existing(**result)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
# import datetime
import random
import uuid

import mock

from openstackclient.tests.unit import utils
from otcextensions.tests.unit.osclient import test_base

from otcextensions.sdk.dcs.v1 import backup
from otcextensions.sdk.dcs.v1 import config
from otcextensions.sdk.dcs.v1 import instance
from otcextensions.sdk.dcs.v1 import restore_record
from otcextensions.sdk.dcs.v1 import statistic
from otcextensions.sdk.dcs.v1 import quota


class TestDCS(utils.TestCommand):

    def setUp(self):
        super(TestDCS, self).setUp()

        self.app.client_manager.dcs = mock.Mock()
        self.client = self.app.client_manager.dcs

        self.client.get_instance = mock.Mock()
        self.client.find_instance = mock.Mock()
        self.client.instances = mock.Mock()
        self.client.delete_instance = mock.Mock()
        self.client.update_instance = mock.Mock()
        self.client.create_instance = mock.Mock()
        self.client.extend_instance = mock.Mock()


class FakeInstance(test_base.Fake):
    """Fake one or more Instance"""

    @classmethod
    def generate(cls):
        object_info = {
            'name': 'group-' + uuid.uuid4().hex,
            'id': 'id-' + uuid.uuid4().hex,
            'description': 'SOME description',
            'status': random.choice(['CREATING', 'CREATEFILED',
                                     'RUNNING', 'ERROR', 'STARTING',
                                     'RESTARTING', 'CLOSING', 'CLOSED',
                                     'EXTENDING']),
            'engine': uuid.uuid4().hex,
            'capacity': random.randint(1, 100),
            'ip': uuid.uuid4().hex,
            'port': random.randint(1, 65535),
            'resource_spec_code': random.choice(['dcs.single_node',
                                                 'dcs.master_standby',
                                                 'dcs.cluster'
                                                 ]),
            'engine_version': uuid.uuid4().hex,
            'internal_version': uuid.uuid4().hex,
            'charging_mode': random.randint(0, 10),
            'vpc_id': uuid.uuid4().hex,
            'vpc_name': uuid.uuid4().hex,
            'subnet_id': uuid.uuid4().hex,
            'subnet_name': uuid.uuid4().hex,
            'subnet_cidr': uuid.uuid4().hex,
            'security_group_id': uuid.uuid4().hex,
            'security_group_name': uuid.uuid4().hex,
            'created_at': uuid.uuid4().hex,
            'error_code': uuid.uuid4().hex,
            'product_id': random.choice(['OTC_DCS_SINGLE',
                                         'OTC_DCS_MS',
                                         'OTC_DCS_CL']),
            'available_zones': uuid.uuid4().hex,
            'max_memory': random.randint(0, 10),
            'used_memory': random.randint(0, 10),
            'user_id': uuid.uuid4().hex,
            'user_name': uuid.uuid4().hex,
            'order_id': uuid.uuid4().hex,
            'maintain_begin': uuid.uuid4().hex,
            'maintain_end': uuid.uuid4().hex,
        }
        obj = instance.Instance.existing(**object_info)
        return obj


class FakeStatistic(test_base.Fake):
    """Fake one or more Statistic"""

    @classmethod
    def generate(cls):
        object_info = {
            'instance_id': 'instance_id-' + uuid.uuid4().hex,
            'max_memory': random.randint(1, 65535),
            'used_memory': random.randint(1, 65535),
            'cmd_get_count': random.randint(1, 65535),
            'cmd_set_count': random.randint(1, 65535),
            'used_cpu': 'cpu-' + uuid.uuid4().hex,
            'input_kbps': 'input-' + uuid.uuid4().hex,
            'output_kbps': 'output-' + uuid.uuid4().hex,

        }
        obj = statistic.Statistic.existing(**object_info)
        return obj


class FakeBackup(test_base.Fake):
    """Fake one or more Backup"""

    @classmethod
    def generate(cls):
        object_info = {
            'instance_id': 'instance_id-' + uuid.uuid4().hex,
            'id': 'id-' + uuid.uuid4().hex,
            'size': random.randint(1, 65535),
            'period': uuid.uuid4().hex,
            'description': uuid.uuid4().hex,
            'progress': uuid.uuid4().hex,
            'created_at': uuid.uuid4().hex,
            'updated_at': uuid.uuid4().hex,
            'type': uuid.uuid4().hex,
            'name': uuid.uuid4().hex,
            'error_code': uuid.uuid4().hex,
            'is_restorable': True,

        }
        obj = backup.Backup.existing(**object_info)
        return obj


class FakeRestoreRecord(test_base.Fake):
    """Fake one or more Restore Records"""

    @classmethod
    def generate(cls):
        object_info = {
            'instance_id': 'instance_id-' + uuid.uuid4().hex,
            'max_memory': random.randint(1, 65535),
            'used_memory': random.randint(1, 65535),
            'cmd_get_count': random.randint(1, 65535),
            'cmd_set_count': random.randint(1, 65535),
            'used_cpu': 'cpu-' + uuid.uuid4().hex,
            'input_kbps': 'input-' + uuid.uuid4().hex,
            'output_kbps': 'output-' + uuid.uuid4().hex
        }
        obj = restore_record.RestoreRecord.existing(**object_info)
        return obj


class FakeConfig(test_base.Fake):
    """Fake one or more Config"""

    @classmethod
    def generate(cls):
        object_info = {
            'instance_id': 'instance_id-' + uuid.uuid4().hex,
            'id': uuid.uuid4().hex,
            'name': uuid.uuid4().hex,
            'value': uuid.uuid4().hex,
            'value_type': uuid.uuid4().hex,
            'value_range': uuid.uuid4().hex,
            'default_value': uuid.uuid4().hex,
            'description': uuid.uuid4().hex
        }
        obj = config.Config.existing(**object_info)
        return obj


class FakeQuota(test_base.Fake):
    """Fake one or more Quota"""

    @classmethod
    def generate(cls):

        qtype = random.choice(['instances', 'ram'])

        def _generate_unit_by_type(qtype):
            return 'null' if qtype == 'instances' else 'GB'

        object_info = {
            'quota': random.randint(1, 65535),
            'used': random.randint(1, 65535),
            'type': qtype,
            'unit': _generate_unit_by_type(qtype),
            'max': random.randint(1, 65535),
            'min': random.randint(1, 10)
        }
        obj = quota.Quota.existing(**object_info)
        return obj



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts devenv v1 action implementations"""
import logging

from osc_lib import exceptions
from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

SORT_ORDER_CHOICES = ["asc", "desc"]

STATUS_CHOICES = [
    "CREATING",
    "CREATE_FAILED",
    "STARTING",
    "RESTARTING",
    "START_FAILED",
    "RUNNING",
    "STOPPING",
    "STOPPED",
    "UNAVAILABLE",
    "DELETED",
    "RESIZING",
    "RESIZE_FAILED",
]

DEVENV_PROFILE_ID_MAP = {
    1: "Multi-Engine 1.0 (python3)-cpu",
    2: "Multi-Engine 1.0 (python3)-gpu",
    3: "Multi-Engine 2.0 (python3)",
}

_formatters = {
    "auto_stop": cli_utils.YamlFormat,
    "created_at": cli_utils.UnixTimestampFormatter,
    "flavor_details": cli_utils.YamlFormat,
    "profile": cli_utils.YamlFormat,
    "spec": cli_utils.YamlFormat,
    "updated_at": cli_utils.UnixTimestampFormatter,
    "user": cli_utils.YamlFormat,
}

AUTO_STOP_CHOICES = (
    "enable",
    "disable",
)


def _get_columns(item):
    column_map = {}
    hidden = ["location", "workspace", "ai_project"]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


def translate_response(func):
    def new(self, *args, **kwargs):
        obj = func(self, *args, **kwargs)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)
        return (display_columns, data)

    new.__name__ = func.__name__
    new.__doc__ = func.__doc__
    return new


class ListDevenvInstances(command.Lister):
    _description = _("List Devenv Instances.")
    columns = (
        "ID",
        "Name",
        "status",
        "Created At",
    )

    def get_parser(self, prog_name):
        parser = super(ListDevenvInstances, self).get_parser(prog_name)
        parser.add_argument(
            "--dev-env-type",
            metavar="<dev_env_type>",
            default="Notebook",
            help=_(
                "Development environment type. Only Notebook is supported. "
                "The first letter must be capitalized."
            ),
        )
        parser.add_argument(
            "--provision-type",
            dest="provision_type",
            help=_("Deployment type. Only Docker is supported."),
        )
        parser.add_argument(
            "--status",
            metavar="<status>",
            type=lambda s: s.upper(),
            choices=STATUS_CHOICES,
            help=_("Filter Devenv Instances by status."),
        )
        parser.add_argument(
            "--sort-by",
            metavar="<sort_by>",
            type=lambda s: s.lower(),
            choices=["name", "creation_timestamp"],
            help=_(
                "Classification standard. The value can be name "
                "or creation_timestamp. By default list is sorted by name."
            ),
        )
        parser.add_argument(
            "--order",
            metavar="{" + ",".join(SORT_ORDER_CHOICES) + "}",
            type=lambda s: s.lower(),
            choices=SORT_ORDER_CHOICES,
            help=_("Sorting order. Default value: asc"),
        )
        parser.add_argument(
            "--offset",
            metavar="<offset>",
            type=int,
            help=_("Start index. The default value is 0."),
        )
        parser.add_argument(
            "--limit",
            metavar="<limit>",
            type=int,
            help=_("Number of returned result records."),
        )
        parser.add_argument(
            "--workspace-id",
            metavar="<workspace_id>",
            help=_(
                "Workspace ID. If no workspace is created, "
                "the default value is 0."
            ),
        )
        parser.add_argument(
            "--ai-project-id",
            metavar="<ai_project_id>",
            dest="ai_project",
            help=_("AI project ID."),
        )
        parser.add_argument(
            "--pool-id",
            metavar="<pool_id>",
            help=_("ID of a dedicated resource pool."),
        )
        parser.add_argument(
            "--show-self",
            action="store_true",
            help=("Only the current user's instanaces will be listed."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        args_list = (
            "dev_env_type",
            "provision_type",
            "status",
            "sort_by",
            "order",
            "offset",
            "limit",
            "workspace_id",
            "show_self",
            "ai_project",
            "pool_id",
        )
        query_params = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                query_params[arg] = val

        data = client.devenv_instances(**query_params)

        formatters = {
            "Created At": cli_utils.UnixTimestampFormatter,
        }

        return (
            self.columns,
            (
                utils.get_item_properties(
                    s, self.columns, formatters=formatters
                )
                for s in data
            ),
        )


class ShowDevenvInstance(command.ShowOne):
    _description = _("Show details of a Devenv Instance.")

    def get_parser(self, prog_name):
        parser = super(ShowDevenvInstance, self).get_parser(prog_name)
        parser.add_argument(
            "instance",
            metavar="<instance>",
            help=_("Devenv Instance ID."),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        return client.find_devenv_instance(
            parsed_args.instance, ignore_missing=False
        )


class CreateDevenvInstance(command.ShowOne):
    _description = _("Create a Devenv Instance.")

    def get_parser(self, prog_name):
        parser = super(CreateDevenvInstance, self).get_parser(prog_name)
        parser.add_argument(
            "name",
            metavar="<name>",
            help=_(
                "Instance name (max 64 characters). The value can contain "
                "letters, digits, hyphens (-), and underscores (_)."
            ),
        )
        parser.add_argument(
            "--profile-id",
            metavar="<profile_id>",
            choices=list(DEVENV_PROFILE_ID_MAP.keys()),
            required=True,
            type=int,
            help=_(
                "Configuration ID. The options are as follows:\n"
                + "\n".join(
                    [
                        f"{key}: {value}"
                        for key, value in DEVENV_PROFILE_ID_MAP.items()
                    ]
                )
            ),
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            help=_("Instance description (max 512 characters.)"),
        )
        parser.add_argument(
            "--flavor",
            metavar="<flavor>",
            required=True,
            help=_("Instance flavor."),
        )
        parser.add_argument(
            "--ai-project-id",
            metavar="<ai_project_id>",
            help=_("AI project ID. This parameter is reserved."),
        )
        parser.add_argument(
            "--workspace-id",
            metavar="<workspace_id>",
            help=_("Workspace. The default workspace ID is 0."),
        )

        parser.add_argument(
            "--storage-type",
            metavar="<storage_type>",
            required=True,
            help=_("Storage type. Only `obs` is supported."),
        )
        parser.add_argument(
            "--storage-path",
            metavar="<storage_path>",
            required=True,
            help=_(
                "The value must be a valid OBS bucket path and end with "
                "a slash (/).\nThe value must be a specific directory in "
                "an OBS bucket rather than the root directory of an OBS "
                "bucket."
            ),
        )

        extended_storage_group = parser.add_argument_group(
            "Extended Storage Parameters",
            description=_(
                "Extended storage supports only obs fs and is "
                "available only for certain dedicated resource "
                "pools."
            ),
        )
        parser._action_groups.insert(2, parser._action_groups.pop())
        extended_storage_group.add_argument(
            "--extended-storage-type",
            metavar="<extended_storage_type>",
            help=_("Storage type. Only `obs` is supported."),
        )
        extended_storage_group.add_argument(
            "--extended-storage-path",
            metavar="<extended_storage_path>",
            help=_(
                "If extended_storage_type is set to obs, this parameter is "
                "mandatory.\nThe value must be a valid OBS bucket path "
                "and end with a slash (/).\nThe value must be a specific "
                "directory in an OBS bucket rather than the root directory "
                "of an OBS bucket."
            ),
        )

        autostop_group = parser.add_argument_group("AutoStop Parameters")
        parser._action_groups.insert(3, parser._action_groups.pop())
        autostop_group.add_argument(
            "--autostop-duration",
            metavar="<autostop_duration>",
            type=int,
            help=_(
                "Running duration, in seconds. The value ranges from "
                "3600 to 86400.\nThe instance will automatically stop "
                "when the running duration is reached.\nAfter this "
                "parameter is set, it is valid for each startup."
            ),
        )
        autostop_group.add_argument(
            "--autostop-prompt",
            metavar="<autostop_prompt>",
            type=bool,
            help=_(
                "Whether to display a prompt again.\nThis parameter is "
                "provided for the console to determine whether to display "
                "a prompt again.\nThe default value is true."
            ),
        )

        pool_group = parser.add_argument_group("Resource pool Parameters")
        parser._action_groups.insert(4, parser._action_groups.pop())
        pool_group.add_argument(
            "--pool-id",
            metavar="<pool_id>",
            help=_("Resource pool ID."),
        )
        pool_group.add_argument(
            "--pool-type",
            metavar="<pool_type>",
            help=_("Resource pool Type."),
        )
        pool_group.add_argument(
            "--pool-name",
            metavar="<pool_name>",
            help=_("Resource pool Name."),
        )

        parser.add_argument(
            "--annotation",
            metavar="<annotation>",
            action=parseractions.KeyValueAction,
            help=_(
                "Custom data to be passed as Map<String:String>.\nData may "
                "be passed as <key>=<value> or JSON.\n"
                "(repeat option to set multiple annotations)"
            ),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        profile_id = DEVENV_PROFILE_ID_MAP[parsed_args.profile_id]

        attrs = {
            "name": parsed_args.name,
            "flavor": parsed_args.flavor,
            "profile_id": profile_id,
            "spec": {
                "storage": {
                    "location": {"path": parsed_args.storage_path},
                    "type": parsed_args.storage_type,
                }
            },
        }
        if parsed_args.description:
            attrs["description"] = parsed_args.description

        if parsed_args.autostop_duration:
            attrs["spec"]["auto_stop"] = {
                "enable": True,
                "duration": parsed_args.autostop_duration,
            }
            if parsed_args.autostop_prompt:
                attrs["auto_stop"]["spec"]["auto_stop"].update(
                    {"prompt": parsed_args.autostop_prompt}
                )

        if parsed_args.workspace_id:
            attrs["workspace"] = {"id": parsed_args.workspace_id}
        if parsed_args.ai_project_id:
            attrs["ai_project"] = {"id": parsed_args.ai_project_id}

        if bool(parsed_args.extended_storage_type) ^ bool(
            parsed_args.extended_storage_path
        ):
            raise exceptions.CommandError(
                "--extended-storage-type and --extended-storage-path "
                "must be given together."
            )

        if parsed_args.extended_storage_path:
            attrs["spec"]["extended_storage"] = {
                "location": {"path": parsed_args.extended_storage_path},
                "type": parsed_args.extended_storage_type,
            }

        if parsed_args.pool_id:
            attrs["pool"] = {"id": parsed_args.pool_id}
            if parsed_args.pool_type:
                attrs["pool"] = {"type": parsed_args.pool_type}
            if parsed_args.pool_name:
                attrs["pool"] = {"name": parsed_args.pool_name}

        if parsed_args.annotation:
            attrs["annotations"] = parsed_args.annotation

        return client.create_devenv_instance(**attrs)


class StartDevenvInstance(command.ShowOne):
    _description = _("Start a Devenv Instance.")

    def get_parser(self, prog_name):
        parser = super(StartDevenvInstance, self).get_parser(prog_name)
        parser.add_argument(
            "instance",
            metavar="<instance>",
            help=_("Devenv Instance name or ID."),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        instance = client.find_devenv_instance(
            parsed_args.instance, ignore_missing=False
        )
        return client.start_devenv_instance(instance.id)


class StopDevenvInstance(command.ShowOne):
    _description = _("Start a Devenv Instance.")

    def get_parser(self, prog_name):
        parser = super(StopDevenvInstance, self).get_parser(prog_name)
        parser.add_argument(
            "instance",
            metavar="<instance>",
            help=_("Devenv Instance name or ID."),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        instance = client.find_devenv_instance(
            parsed_args.instance, ignore_missing=False
        )
        return client.stop_devenv_instance(instance.id)


class UpdateDevenvInstance(command.ShowOne):
    _description = _("Update Devenv Description")

    def get_parser(self, prog_name):
        parser = super(UpdateDevenvInstance, self).get_parser(prog_name)
        parser.add_argument(
            "instance",
            metavar="<instance>",
            help=_("Devenv Instance ID."),
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            help=_(
                "Additional service attribute, which facilitates service "
                "management."
            ),
        )
        parser.add_argument(
            "--auto-stop",
            metavar="{" + ",".join(AUTO_STOP_CHOICES) + "}",
            type=lambda s: s.lower(),
            choices=AUTO_STOP_CHOICES,
            help=_("Whether to enable or disable the auto stop function."),
        )
        parser.add_argument(
            "--duration",
            metavar="<duration>",
            type=int,
            help=_(
                "Running duration, in seconds. The value ranges from "
                "3,600 to 86,400. This parameter is mandatory when "
                "auto_stop is enabled."
            ),
        )
        parser.add_argument(
            "--prompt",
            metavar="{" + ",".join(AUTO_STOP_CHOICES) + "}",
            type=lambda s: s.lower(),
            choices=AUTO_STOP_CHOICES,
            help=_(
                "Whether to disable a display prompt again for auto_stop. "
                "This parameter is provided for the console to determine "
                "whether to display a prompt again."
            ),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        attrs = {"auto_stop": {}}
        auto_stop = parsed_args.auto_stop
        prompt = parsed_args.prompt

        if auto_stop:
            if auto_stop == "enable":
                attrs["auto_stop"].update(enable=True)
                if not parsed_args.duration:
                    raise exceptions.CommandError(
                        "--duration is mandatory when --auto-stop is set."
                    )
            elif auto_stop == "disable":
                attrs["auto_stop"].update(enable=False)

        if prompt:
            if prompt == "enable":
                attrs["auto_stop"].update(prompt=True)
            elif prompt == "disable":
                attrs["auto_stop"].update(prompt=False)

        if parsed_args.duration:
            attrs["auto_stop"].update(duration=parsed_args.duration)

        if parsed_args.description:
            attrs.update(description=parsed_args.description)

        if not attrs["auto_stop"]:
            del attrs["auto_stop"]
        return client.update_devenv_instance(parsed_args.instance, **attrs)


class DeleteDevenvInstance(command.Command):
    _description = _("Delete Devenv Instance(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteDevenvInstance, self).get_parser(prog_name)
        parser.add_argument(
            "instance",
            metavar="<instance>",
            nargs="+",
            help=_("ID or Name of the Devenv Instance(s) to be deleted."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        result = 0
        for name_or_id in parsed_args.instance:
            try:
                instance = client.find_devenv_instance(
                    name_or_id, ignore_missing=False
                )
                client.delete_devenv_instance(instance.id)
            except Exception as e:
                result += 1
                LOG.error(
                    _(
                        "Failed to delete Devenv Instance(s) with "
                        "ID or Name '%(instance)s': %(e)s"
                    ),
                    {"instance": name_or_id, "e": e},
                )
        if result > 0:
            total = len(parsed_args.instance)
            msg = _(
                "%(result)s of %(total)s Devenv Instance(s) failed "
                "to delete."
            ) % {"result": result, "total": total}
            raise exceptions.CommandError(msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from otcextensions.osclient.dms.v1 import instance
from otcextensions.tests.unit.osclient.dms.v1 import fakes


class TestDMSInstance(fakes.TestDMS):

    def setUp(self):
        super(TestDMSInstance, self).setUp()
        self.client = self.app.client_manager.dms


class TestListDMSInstance(TestDMSInstance):

    instances = fakes.FakeInstance.create_multiple(3)

    columns = ('ID', 'name', 'engine_name', 'engine_version',
               'storage_spec_code', 'status', 'connect_address',
               'router_id', 'network_id', 'security_group_id',
               'user_name', 'storage', 'total_storage', 'used_storage')

    data = []

    for s in instances:
        data.append((
            s.id,
            s.name,
            s.engine_name,
            s.engine_version,
            s.storage_spec_code,
            s.status,
            s.connect_address,
            s.router_id,
            s.network_id,
            s.security_group_id,
            s.user_name,
            s.storage,
            s.total_storage,
            s.used_storage
        ))

    def setUp(self):
        super(TestListDMSInstance, self).setUp()

        self.cmd = instance.ListDMSInstance(self.app, None)

        self.client.instances = mock.Mock()

    def test_list(self):
        arglist = [
        ]

        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.instances.side_effect = [
            self.instances
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.instances.assert_called_once_with(include_failure=False)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--engine-name', 'engine',
            '--status', 'Creating',
            '--include-failure'
        ]

        verifylist = [
            ('engine_name', 'engine'),
            ('status', 'CREATING'),
            ('include_failure', True)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.instances.side_effect = [
            self.instances
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.instances.assert_called_once_with(
            engine_name='engine',
            status='CREATING',
            include_failure=True)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowDMSInstance(TestDMSInstance):

    _data = fakes.FakeInstance.create_one()

    columns = ('access_user', 'availability_zones', 'description',
               'engine_name', 'engine_version', 'is_public', 'is_ssl',
               'kafka_public_status', 'maintenance_end', 'name', 'network_id',
               'password', 'product_id', 'public_bandwidth',
               'retention_policy', 'router_id', 'router_name',
               'security_group_id',
               'security_group_name', 'storage', 'storage_spec_code')

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowDMSInstance, self).setUp()

        self.cmd = instance.ShowDMSInstance(self.app, None)

        self.client.find_instance = mock.Mock()

    def test_show_default(self):
        arglist = [
            'test_instance'
        ]
        verifylist = [
            ('instance', 'test_instance')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_instance.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteDMSInstance(TestDMSInstance):

    def setUp(self):
        super(TestDeleteDMSInstance, self).setUp()

        self.cmd = instance.DeleteDMSInstance(self.app, None)

        self.client.delete_instance = mock.Mock()

    def test_delete(self):
        arglist = ['t1']
        verifylist = [
            ('instance', ['t1'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_instance.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [mock.call('t1')]

        self.client.delete_instance.assert_has_calls(calls)
        self.assertEqual(1, self.client.delete_instance.call_count)

    def test_delete_multiple(self):
        arglist = [
            't1',
            't2',
        ]
        verifylist = [
            ('instance', ['t1', 't2'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_instance.side_effect = [{}, {}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [mock.call('t1'), mock.call('t2')]

        self.client.delete_instance.assert_has_calls(calls)
        self.assertEqual(2, self.client.delete_instance.call_count)


class TestCreateDMSInstance(TestDMSInstance):

    _data = fakes.FakeInstance.create_one()

    columns = ('access_user', 'availability_zones', 'description',
               'engine_name', 'engine_version', 'is_public', 'is_ssl',
               'kafka_public_status', 'maintenance_end', 'name', 'network_id',
               'password',
               'product_id', 'public_bandwidth', 'retention_policy',
               'router_id', 'router_name', 'security_group_id',
               'security_group_name', 'storage', 'storage_spec_code')

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateDMSInstance, self).setUp()

        self.cmd = instance.CreateDMSInstance(self.app, None)

        self.client.create_instance = mock.Mock()
        self.app.client_manager.network = mock.Mock()
        self.app.client_manager.network.find_router = mock.Mock()
        self.app.client_manager.network.find_network = mock.Mock()
        self.app.client_manager.compute = mock.Mock()

    def test_create_default(self):
        arglist = [
            'name',
            '--description', 'descr',
            '--engine-name', 'kafka',
            '--engine-version', '2.1.0',
            '--storage', '15',
            '--access-user', 'u1',
            '--password', 'pwd',
            '--router', 'router_id',
            '--security-group', 'sg_id',
            '--network', 'net_id',
            '--availability-zone', 'az1',
            '--availability-zone', 'az2',
            '--product-id', 'pid',
            '--maintenance-begin', 'mwb',
            '--maintenance-end', 'mwe',
            '--enable-public-access',
            '--enable-ssl',
            '--public-bandwidth', '14',
            '--retention-policy', 'produce_reject',
            '--storage-spec-code', 'dms.physical.storage.high'
        ]
        verifylist = [
            ('name', 'name'),
            ('description', 'descr'),
            ('engine_name', 'kafka'),
            ('engine_version', '2.1.0'),
            ('storage', 15),
            ('access_user', 'u1'),
            ('password', 'pwd'),
            ('router', 'router_id'),
            ('security_group', 'sg_id'),
            ('network', 'net_id'),
            ('availability_zone', ['az1', 'az2']),
            ('product_id', 'pid'),
            ('maintenance_begin', 'mwb'),
            ('maintenance_end', 'mwe'),
            ('enable_public_access', True),
            ('enable_ssl', True),
            ('public_bandwidth', 14),
            ('retention_policy', 'produce_reject'),
            ('storage_spec_code', 'dms.physical.storage.high')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_instance.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.app.client_manager.network.find_router.assert_called_with(
            'router_id', ignore_missing=False)
        self.app.client_manager.network.find_network.assert_called_with(
            'net_id', ignore_missing=False)
        self.app.client_manager.compute.find_security_group.assert_called_with(
            'sg_id', ignore_missing=False)

        self.client.create_instance.assert_called_with(
            access_user='u1',
            availability_zone=['az1', 'az2'],
            description='descr',
            engine_name='kafka',
            engine_version='2.1.0',
            is_public=True,
            is_ssl=True,
            maintenance_begin='mwb',
            maintenance_end='mwe',
            name='name',
            password='pwd',
            product_id='pid',
            public_bandwidth=14,
            retention_policy='produce_reject',
            router_id=mock.ANY,
            security_group_id=mock.ANY,
            storage=15,
            storage_spec_code='dms.physical.storage.high',
            network_id=mock.ANY
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestUpdateDMSInstance(TestDMSInstance):

    _data = fakes.FakeInstance.create_one()

    columns = ('access_user', 'availability_zones', 'description',
               'engine_name', 'engine_version', 'is_public', 'is_ssl',
               'kafka_public_status', 'maintenance_end', 'name', 'network_id',
               'password',
               'product_id', 'public_bandwidth', 'retention_policy',
               'router_id', 'router_name', 'security_group_id',
               'security_group_name', 'storage', 'storage_spec_code')

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestUpdateDMSInstance, self).setUp()

        self.cmd = instance.UpdateDMSInstance(self.app, None)

        self.client.update_instance = mock.Mock()
        self.client.find_instance = mock.Mock(return_value=self._data)
        self.app.client_manager.compute = mock.Mock()

    def test_update_default(self):
        arglist = [
            'inst',
            '--name', 'new_name',
            '--description', 'descr',
            '--security-group', 'sg_id',
            '--maintenance-begin', 'mwb',
            '--maintenance-end', 'mwe'
        ]
        verifylist = [
            ('instance', 'inst'),
            ('name', 'new_name'),
            ('description', 'descr'),
            ('security_group', 'sg_id'),
            ('maintenance_begin', 'mwb'),
            ('maintenance_end', 'mwe'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_instance.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with(
            'inst', ignore_missing=False)
        self.app.client_manager.compute.find_security_group.assert_called_with(
            'sg_id', ignore_missing=False)

        self.client.update_instance.assert_called_with(
            self._data,
            description='descr',
            maintenance_begin='mwb',
            maintenance_end='mwe',
            name='new_name',
            security_group_id=mock.ANY
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestRestartDMSInstance(TestDMSInstance):

    _data = fakes.FakeInstance.create_one()

    def setUp(self):
        super(TestRestartDMSInstance, self).setUp()

        self.cmd = instance.RestartDMSInstance(self.app, None)

        self.client.find_instance = mock.Mock()
        self.client.restart_instance = mock.Mock()

    def test_show_default(self):
        arglist = [
            'test_instance'
        ]
        verifylist = [
            ('instance', 'test_instance')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_instance.side_effect = [
            self._data
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with('test_instance',
                                                     ignore_missing=False)
        self.client.restart_instance.assert_called_with(self._data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.nat.v2 import gateway


INSTANCE_NAME = 'GATEWAYNAME'
INSTANCE_ID = 'a78fb3eb-1654-4710-8742-3fc49d5f04f8'
EXAMPLE = {
    'router_id': 'd84f345c-80a1-4fa2-a39c-d0d397c3f09a',
    'status': 'PENDING_CREATE',
    'description': 'Test Gateway Response',
    'admin_state_up': True,
    'project_id': '27e25061336f4af590faeabeb7fcd9a3',
    'created_at': '2017-11-18 07:34:32.203044',
    'spec': '2',
    'internal_network_id': '89d66639-aacb-4929-969d-07080b0f9fd9',
    'id': INSTANCE_ID,
    'name': INSTANCE_NAME
}


class TestGateway(base.TestCase):

    def test_basic(self):
        sot = gateway.Gateway()
        self.assertEqual('nat_gateway', sot.resource_key)
        self.assertEqual('nat_gateways', sot.resources_key)
        path = '/nat_gateways'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = gateway.Gateway(**EXAMPLE)
        self.assertEqual(EXAMPLE['router_id'], sot.router_id)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['admin_state_up'], sot.admin_state_up)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['spec'], sot.spec)
        self.assertEqual(EXAMPLE['internal_network_id'],
                         sot.internal_network_id)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts service v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

INFER_TYPE_CHOICES = ["real-time", "batch"]


class ListJobEngines(command.Lister):
    _description = _("List job engine Specifications.")
    columns = (
        "Engine Id",
        "Engine Name",
        "Engine Type",
        "Engine Version",
    )

    def get_parser(self, prog_name):
        parser = super(ListJobEngines, self).get_parser(prog_name)
        parser.add_argument(
            "--job-type",
            metavar="{train, inference}",
            type=lambda s: s.lower(),
            choices=["train", "inference"],
            help=_("Job Type."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        query_params = {}
        if parsed_args.job_type:
            query_params["job_type"] = parsed_args.job_type

        data = client.job_engines(**query_params)

        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import openstack
import uuid

from otcextensions.tests.functional import base

_logger = openstack._log.setup_logging('openstack')


class TestGateway(base.BaseFunctionalTest):
    uuid_v4 = uuid.uuid4().hex[:8]
    update_gateway_name = uuid_v4 + 'update-test-gateway'
    network_info = None
    gateway = None
    gateway_name = uuid_v4 + 'test-gateway-gateway'
    attrs = {
        "name": gateway_name,
        "spec": "1"
    }

    def create_network(self):
        cidr = '192.168.0.0/16'
        ipv4 = 4
        uuid_v4 = uuid.uuid4().hex[:8]
        router_name = 'gw-test-router-' + uuid_v4
        net_name = 'gw-test-net-' + uuid_v4
        subnet_name = 'gw-test-subnet-' + uuid_v4

        if not TestGateway.network_info:
            network = self.conn.network.create_network(name=net_name)
            self.assertEqual(net_name, network.name)
            net_id = network.id
            subnet = self.conn.network.create_subnet(
                name=subnet_name,
                ip_version=ipv4,
                network_id=net_id,
                cidr=cidr
            )
            self.assertEqual(subnet_name, subnet.name)
            subnet_id = subnet.id

            router = self.conn.network.create_router(name=router_name)
            self.assertEqual(router_name, router.name)
            router_id = router.id
            interface = router.add_interface(
                self.conn.network,
                subnet_id=subnet_id
            )
            self.assertEqual(interface['subnet_id'], subnet_id)
            self.assertIn('port_id', interface)

            TestGateway.network_info = {
                'router_id': router_id,
                'subnet_id': subnet_id,
                'network_id': net_id
            }
        if not TestGateway.gateway:
            self.attrs['router_id'] = TestGateway.network_info['router_id']
            self.attrs['internal_network_id'] = \
                TestGateway.network_info['network_id']
            TestGateway.gateway = self.conn.nat.create_gateway(**self.attrs)
            self.conn.nat.wait_for_gateway(TestGateway.gateway)
            self.assertIsNotNone(TestGateway.gateway)

    def destroy_network(self):
        if TestGateway.network_info:
            router_id = TestGateway.network_info['router_id']
            subnet_id = TestGateway.network_info['subnet_id']
            network_id = TestGateway.network_info['network_id']
            router = self.conn.network.get_router(router_id)

            interface = router.remove_interface(
                self.conn.network,
                subnet_id=subnet_id
            )
            self.assertEqual(interface['subnet_id'], subnet_id)
            self.assertIn('port_id', interface)
            sot = self.conn.network.delete_router(
                router_id,
                ignore_missing=False
            )
            self.assertIsNone(sot)
            sot = self.conn.network.delete_subnet(
                subnet_id,
                ignore_missing=False
            )
            self.assertIsNone(sot)
            sot = self.conn.network.delete_network(
                network_id,
                ignore_missing=False
            )

            TestGateway.network_info = None
            self.assertIsNone(sot)

    def test_01_list_gateways(self):
        self.create_network()
        self.gateways = list(self.conn.nat.gateways())
        self.assertGreaterEqual(len(self.gateways), 0)

    def test_02_get_gateway(self):
        gateway = self.conn.nat.get_gateway(TestGateway.gateway.id)
        self.assertEqual(gateway.name, self.gateway_name)

    def test_03_find_gateway(self):
        gateway = self.conn.nat.find_gateway(TestGateway.gateway.name)
        self.assertEqual(gateway.name, self.gateway_name)

    def test_04_update_gateway(self):
        update_gw = self.conn.nat.update_gateway(
            gateway=TestGateway.gateway.id,
            name=self.update_gateway_name)
        update_gw = self.conn.nat.get_gateway(update_gw.id)
        self.assertEqual(update_gw.name, self.update_gateway_name)

    def test_05_delete_gateway(self):
        self.conn.nat.delete_gateway(gateway=TestGateway.gateway)
        self.conn.nat.wait_for_delete_gateway(TestGateway.gateway)
        TestGateway.gateway = None
        self.destroy_network()
        gateway = self.conn.nat.find_gateway(
            self.update_gateway_name, ignore_missing=True)
        self.assertIsNone(gateway)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions

from otcextensions.common import exc


class Smn(resource.Resource):
    #: Specifies whether SMN is supported.
    enabled = resource.Body('is_support_smn', type=bool)
    #: Specifies the theme of the SMN service.
    topic_id = resource.Body('topic_id')
    #: * Specifies trigger conditions for sending a notification when `Typical`
    #: is selected. You can select Delete, Create, or Login or at least one
    #: of them.
    #: * Specifies trigger conditions for sending a notification when `All`
    #: is selected. All conditions including Delete, Create, Change, and
    #: OpenStack API Event are selected by default. Modification is not
    #: allowed.
    operations = resource.Body('operations', type=list)
    #: You can select Typical or All for Trigger Condition.
    #: * When the value is `false`, operations cannot be left empty.
    #: * When the value is `true`, operations is not supported.
    is_send_all_key_operation = resource.Body(
        'is_send_all_key_operation', type=bool)
    #: In Typical scenario, you can specify the users using the login function.
    #: When these users log in, notifications will be sent.
    #: * After this function is enabled, the value is the list of the
    #: specified users. Separate them with commas (,). A maximum of 50
    #: users is supported.
    #: * If the value is null, the target objects are all users under the
    #: current tenant by default.
    notify_users = resource.Body('need_notify_user_list', type=list)


class Tracker(resource.Resource):

    base_path = '/tracker'

    _query_mapping = resource.QueryParameters('tracker_name')

    # capabilities
    allow_create = True
    allow_commit = True
    allow_fetch = True
    allow_delete = True

    # Properties
    #: tracker name
    name = resource.Body('tracker_name', alternate_id=True)
    #: bucket name
    bucket_name = resource.Body('bucket_name')
    #: file prefix name in a bucket
    file_prefix_name = resource.Body('file_prefix_name')
    #: Status of the tracker
    #: values: `enabled`, `disabled`, `error`
    status = resource.Body('status')
    #: Detail of the tracker, only validate if exception happens
    detail = resource.Body('detail')
    #: SMN
    smn = resource.Body('smn', type=Smn)

    def _delete_tracker(self, session, tracker):
        """Delete Tracker
        """
        url = self.base_path + '?tracker_name={}'.format(tracker)
        response = session.delete(url)
        exceptions.raise_from_response(response)
        return None

    def _translate_response(self, response, has_body=None, error_message=None,
                            resource_response_key=None):
        """Given a KSA response, inflate this instance with its data

        'DELETE' operations don't return a body, so only try to work
        with a body when has_body is True.

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body
        exc.raise_from_response(response, error_message=error_message)
        if has_body:
            if response.status_code == 204:
                # Some bad APIs (i.e. DCS.Backup.List) return emptiness
                self.log.warn('API returned no content, while it was expected')
                return
            # NOTE: in difference to doc "GET" method return list with 1 item
            body = response.json()
            if isinstance(body, list):
                body = body[0]
            if self.resource_key and self.resource_key in body:
                body = body[self.resource_key]

            body = self._consume_body_attrs(body)
            self._body.attributes.update(body)
            self._body.clean()

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils


class Datastore(resource.Resource):
    base_path = '/datastores'

    # capabilities
    allow_list = True

    datastore_name = resource.URI('datastore_name')

    #: Storage engine
    storage_engine = resource.Body('storage_engine')
    #: Datastore type
    type = resource.Body('type')
    # Indicates the database version.
    # :*Type:string*
    version = resource.Body('version')

    @classmethod
    def list(cls, session, paginated=True, base_path=None,
             allow_unknown_params=False, **params):

        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path

        datastore_type = params.get('datastore_name', 'DDS-Community')
        uri = utils.urljoin(
            base_path,
            datastore_type,
            'versions'
        )

        response = session.get(
            uri,
            headers={"Accept": "application/json"},
            microversion=microversion)
        exceptions.raise_from_response(response)
        data = response.json()

        resources = data['versions']

        if not isinstance(resources, list):
            resources = [resources]

        for raw_resource in resources:
            value = cls.existing(
                microversion=microversion,
                connection=session._get_connection(),
                type=datastore_type,
                storage_engine='wiredTiger',
                version=raw_resource)
            yield value



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid
import json

from openstackclient.tests.functional import base


class TestCce(base.TestCase):
    """Functional tests for CCE Instance. """

    UUID = uuid.uuid4().hex[:4]
    ROUTER_NAME = 'sdk_cce_test_router' + UUID
    NET_NAME = 'sdk_cce_test_net' + UUID
    SUBNET_NAME = 'sdk_cce_test_subnet' + UUID
    ROUTER_ID = None
    NET_ID = None

    CLUSTER_NAME = 'sdk-test-cce' + UUID

    def test_01_cluster_list(self):
        self.openstack(
            'cce cluster list -f json '
        )

    def test_02_cluster_create(self):
        self._initialize_network()
        cmd = (
            f'cce cluster create '
            f'{self.CLUSTER_NAME} '
            f'{self.ROUTER_NAME} '
            f'{self.NET_NAME} '
            f'--description descr '
            f'--flavor cce.s1.small '
            f'--container-network-mode overlay_l2 '
            f'--wait --wait-interval 10 '
            f'-f json'
        )

        json_output = json.loads(self.openstack(cmd))

        self.assertIsNotNone(json_output['ID'])

    def test_03_cluster_delete(self):
        self.addCleanup(self._deinitialize_network)
        self.openstack(
            f'cce cluster delete {self.CLUSTER_NAME} --wait --wait-interval 10'
        )

    def _initialize_network(self):
        router = json.loads(self.openstack(
            'router create -f json ' + self.ROUTER_NAME
        ))
        self.assertIsNotNone(router)

        net = json.loads(self.openstack(
            'network create -f json ' + self.NET_NAME
        ))
        self.assertIsNotNone(net)

        subnet = json.loads(self.openstack(
            f'subnet create {self.SUBNET_NAME} '
            f'-f json --network {self.NET_NAME} '
            f'--subnet-range 192.168.0.0/24'
        ))
        self.assertIsNotNone(subnet)

        self.openstack(
            f'router add subnet {self.ROUTER_NAME} {self.SUBNET_NAME}'
        )

        self.ROUTER_ID = router['id']
        self.NET_ID = net['id']

    def _deinitialize_network(self):
        self.openstack(
            f'router remove subnet {self.ROUTER_NAME} {self.SUBNET_NAME}')
        self.openstack('subnet delete ' + self.SUBNET_NAME)
        self.openstack('network delete ' + self.NET_NAME)
        self.openstack('router delete ' + self.ROUTER_NAME)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import _log

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestServiceSpecification(base.BaseFunctionalTest):

    def setUp(self):
        super(TestServiceSpecification, self).setUp()
        self.dcs = self.conn.dcs

    def test_service_specification(self):
        objects = list(self.dcs.service_specifications())

        self.assertGreaterEqual(len(objects), 1)

        for obj in objects:
            self.assertIsInstance(obj, dict)
            self.assertIn("details", obj.keys())
            self.assertIn("engine", obj.keys())
            self.assertIn("price", obj.keys())
            self.assertIn("currency", obj.keys())
            self.assertIn("flavors", obj.keys())
            self.assertIn("product_id", obj.keys())
            self.assertIn("spec_code", obj.keys())
            self.assertIn("cache_mode", obj.keys())
            self.assertIn("product_type", obj.keys())
            self.assertIn("cpu_type", obj.keys())
            self.assertIn("storage_type", obj.keys())
            self.assertIn("engine_versions", obj.keys())
            self.assertIn("spec_details", obj.keys())
            self.assertIn("spec_details2", obj.keys())
            self.assertIn("charging_type", obj.keys())
            self.assertIn("prod_type", obj.keys())
            self.assertIn("cloud_service_type_code", obj.keys())
            self.assertIn("cloud_resource_type_code", obj.keys())

    def test_service_specification_details(self):
        objects = list(self.dcs.service_specifications())
        details = []

        for obj in objects:
            details.append(obj["details"])

        for obj in details:
            self.assertIsInstance(obj, dict)
            self.assertIn("capacity", obj.keys())
            self.assertIn("max_bandwidth", obj.keys())
            self.assertIn("max_clients", obj.keys())
            self.assertIn("max_connections", obj.keys())
            self.assertIn("max_in_bandwidth", obj.keys())
            self.assertIn("max_memory", obj.keys())
            self.assertIn("tenant_ip_count", obj.keys())
            self.assertIn("sharding_num", obj.keys())
            self.assertIn("proxy_num", obj.keys())
            self.assertIn("db_number", obj.keys())

    def test_service_specification_flavors(self):
        objects = list(self.dcs.service_specifications())
        flavors = []

        for obj in objects:
            flavors.append(obj["flavors"])

        for obj in flavors:
            self.assertIsInstance(obj, list)
            for item in obj:
                self.assertIn("capacity", item.keys())
                self.assertIn("unit", item.keys())
                self.assertIn("available_zones", item.keys())



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""DWS Flavors (Node Types) action implementation"""

import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class ListFlavors(command.Lister):
    _description = _('List Flavors (Node Types) of a DWS Cluster')
    columns = (
        'ID',
        'Name',
        'Availability Zones',
        'vCPU',
        'RAM',
        'Disk Size',
        'Disk Type',
    )

    def take_action(self, parsed_args):
        client = self.app.client_manager.dws
        data = client.flavors()
        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base
from otcextensions.sdk.ctsv3.v3 import key_event

EXAMPLE = {
    "notification_name": "test",
    "operation_type": "complete",
    "topic_id": "urn:smn:{regionid}:24edf66e79d04187acb99a463e610764:test"
}


class TestKeyEvent(base.TestCase):
    def test_basic(self):
        sot = key_event.KeyEvent()
        self.assertEqual('/notifications', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = key_event.KeyEvent(**EXAMPLE)
        self.assertEqual(EXAMPLE['notification_name'], sot.notification_name)
        self.assertEqual(EXAMPLE['operation_type'], sot.operation_type)
        self.assertEqual(EXAMPLE['topic_id'], sot.topic_id)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CES Alarm v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class ListMetrics(command.Lister):
    _description = _('List CES Metrics')
    columns = (
        'namespace',
        'dimensions',
        'metric_name',
        'unit',
    )

    table_columns = (
        'namespace',
        'dimensions.name',
        'dimensions.value',
        'metric_name',
        'unit',
    )

    def get_parser(self, prog_name):
        parser = super(ListMetrics, self).get_parser(prog_name)

        parser.add_argument(
            '--namespace',
            metavar='<namespace>',
            help=_('Namespace of the monitored object, e.g.\n'
                   'SYS.ECS, SYS.VPC')
        )
        parser.add_argument(
            '--metric-name',
            metavar='<metric_name>',
            help=_('Name of the metrics object.')
        )
        parser.add_argument(
            '--unit',
            metavar='<unit>',
            help=_('Unit which is measured.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.ces

        query = {}

        if parsed_args.namespace:
            query['namespace'] = parsed_args.namespace
        if parsed_args.metric_name:
            query['metric_name'] = parsed_args.metric_name
        if parsed_args.unit:
            query['unit'] = parsed_args.unit

        data = client.metrics(**query)

        # Modify table output to provide a better metric overview.
        # Given data set is taken, splitted and flattened to build the table.
        table = ()
        temp_list = []
        big_list = []
        for s in data:
            for item in utils.get_dict_properties(s, self.columns):
                if isinstance(item, (list)):
                    temp_list.append(item[0].name)
                    temp_list.append(item[0].value)
                else:
                    temp_list.append(item)
            big_list.append(tuple(temp_list))
            temp_list = []

        table = (self.table_columns, big_list)

        return table



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import print_function

import collections
import hashlib
# import errno
# import functools
# import json
import os
# import re
# import sys
import re
import threading
from urllib import parse

# import uuid
#
from urllib.parse import urlsplit

if os.name == 'nt':
    pass
else:
    msvcrt = None

# from oslo_utils import encodeutils
# from oslo_utils import strutils
# import prettytable
# import wrapt


_memoized_property_lock = threading.Lock()

SENSITIVE_HEADERS = ('X-Auth-Token',)
REQUIRED_FIELDS_ON_DATA = ('disk_format', 'container_format')


def merge_two_dicts(x, y):
    """Given two dicts, merge them into a new dict as a shallow copy."""
    z = x.copy()
    if y:
        z.update(y)
    return z


def _calculate_data_hashes(data):
    _md5 = hashlib.md5(usedforsecurity=False)

    if hasattr(data, 'read'):
        for chunk in iter(lambda: data.read(8192), b''):
            _md5.update(chunk)
    else:
        _md5.update(data)
    return _md5.hexdigest()


def _get_file_hashes(filename):
    _md5 = None
    with open(filename, 'rb') as file_obj:
        _md5 = _calculate_data_hashes(file_obj)

    return _md5


class FileSegment:
    """File-like object to pass to requests."""

    def __init__(self, filename, offset, length):
        self.filename = filename
        self.offset = offset
        self.length = length
        self.pos = 0
        self._file = open(filename, 'rb')
        self.seek(0)

    def tell(self):
        return self._file.tell() - self.offset

    def seek(self, offset, whence=0):
        if whence == 0:
            self._file.seek(self.offset + offset, whence)
        elif whence == 1:
            self._file.seek(offset, whence)
        elif whence == 2:
            self._file.seek(self.offset + self.length - offset, 0)

    def read(self, size=-1):
        remaining = self.length - self.pos
        if remaining <= 0:
            return b''

        to_read = remaining if size < 0 else min(size, remaining)
        chunk = self._file.read(to_read)
        self.pos += len(chunk)

        return chunk

    def reset(self):
        self._file.seek(self.offset, 0)


def _get_file_segments(endpoint, filename, file_size, segment_size):
    # Use an ordered dict here so that testing can replicate things
    segments = collections.OrderedDict()
    for (index, offset) in enumerate(range(0, file_size, segment_size)):
        remaining = file_size - (index * segment_size)
        segment = FileSegment(
            filename, offset,
            segment_size if segment_size < remaining else remaining)
        name = '{endpoint}/{index}'.format(
            endpoint=endpoint, index=index + 1)
        segments[name] = segment
    return segments


def extract_region_from_url(url):
    parsed_url = urlsplit(url)
    hostname = parsed_url.hostname
    match = re.search(r'eu-[^.]+', hostname)

    if match:
        return match.group()
    return


def extract_url_parts(url: str, project_id: str) -> list:
    """
    Extracts meaningful parts of a URL, excluding the project_id,
    version identifiers, and empty segments.

    Args:
        url (str): The URL to process.
        project_id (str): The project ID to exclude from the URL parts.

    Returns:
        list: A list of meaningful URL segments.
    """
    # Parse and strip the URL path
    path = parse.urlparse(url).path.strip()

    # Remove leading '/' to keep list indexes consistent
    if path.startswith('/'):
        path = path[1:]

    # Split the path into parts, excluding the project_id
    url_parts = [x for x in path.split('/') if x != project_id]

    # Exclude parts that are version identifiers
    url_parts = list(filter(
        lambda x: len(x) > 0 and not
        (x.lower().startswith('v') and x[1:].isdigit()),
        url_parts
    ))

    # Strip out empty or None segments and return
    return [part for part in url_parts if part]



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import openstack
import uuid

from otcextensions.tests.functional import base

_logger = openstack._log.setup_logging('openstack')


class TestContainer(base.BaseFunctionalTest):
    uuid_v4 = uuid.uuid4().hex[:8]
    bucket_name = 'obs-test-' + uuid_v4
    container = None

    def setUp(self):
        super(TestContainer, self).setUp()
        self.client = self.conn.obs
        self.container = self.client.create_container(
            name=self.bucket_name,
            storage_acl='public-read-write',
            storage_class='STANDARD'
        )
        self.addCleanup(self.client.delete_container, self.container)

    def test_01_get_container(self):
        bucket = self.client.get_container(self.bucket_name)
        self.assertIsNotNone(bucket)
        self.assertIsNotNone(bucket.name, self.bucket_name)

    def test_02_list_containers(self):
        containers = list(self.client.containers())
        self.assertIsNotNone(containers)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import exceptions
from openstack import resource


class Quota(resource.Resource):
    base_path = '/quotas'

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'type',
    )

    # Properties
    #: Specifies the maximum quota of resources.
    quota = resource.Body('quota', type=int)
    #: Specifies the resource type.
    type = resource.Body('type')
    #: Specifies the number of created resources.
    used = resource.Body('used', type=int)

    @classmethod
    def list(cls, session, paginated=False, base_path=None, **params):
        """This method is a generator which yields resource objects.

        This resource object list generator handles pagination and takes query
        params for response filtering.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param bool paginated: ``True`` if a GET to this resource returns
            a paginated series of responses, or ``False`` if a GET returns only
            one page of data. **When paginated is False only one page of data
            will be returned regardless of the API's support of pagination.**
        :param str base_path: Base part of the URI for listing resources, if
            different from :data:`~openstack.resource.Resource.base_path`.
        :param bool allow_unknown_params: ``True`` to accept, but discard
            unknown query parameters. This allows getting list of 'filters' and
            passing everything known to the server. ``False`` will result in
            validation exception when unknown query parameters are passed.
        :param str microversion: API version to override the negotiated one.
        :param dict params: These keyword arguments are passed through the
            :meth:`~openstack.resource.QueryParamter._transpose` method
            to find if any of them match expected query parameters to be sent
            in the *params* argument to
            :meth:`~keystoneauth1.adapter.Adapter.get`. They are additionally
            checked against the :data:`~openstack.resource.Resource.base_path`
            format string to see if any path fragments need to be filled in by
            the contents of this argument.
            Parameters supported as filters by the server side are passed in
            the API call, remaining parameters are applied as filters to the
            retrieved results.

        :return: A generator of :class:`Resource` objects.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
            :data:`Resource.allow_list` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.InvalidResourceQuery` if query
            contains invalid params.
        """

        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path
        cls._query_mapping._validate(params, base_path=base_path)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = base_path % params

        # Copy query_params due to weird mock unittest interactions
        response = session.get(
            uri,
            headers={"Accept": "application/json"},
            params=query_params.copy(),
            microversion=microversion,
        )
        exceptions.raise_from_response(response)
        data = response.json()

        quotas_data = data.get('quotas')
        if not quotas_data:
            return

        resources = quotas_data.get('resources')
        if not resources:
            return

        if not isinstance(resources, list):
            resources = [resources]

        for raw_resource in resources:
            raw_resource.pop("self", None)
            value = cls.existing(
                microversion=microversion,
                connection=session._get_connection(),
                **raw_resource
            )
            yield value



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from otcextensions.sdk.function_graph.v2 import function

from openstack import _log

from otcextensions.tests.functional.sdk.function_graph import TestFg

_logger = _log.setup_logging('openstack')


class TestFunctionInstances(TestFg):
    ID = None
    uuid = uuid.uuid4().hex[:8]

    def setUp(self):
        super(TestFunctionInstances, self).setUp()
        self.function = self.client.create_function(**TestFg.function_attrs)
        assert isinstance(self.function, function.Function)

        attrs = {
            "count": 2
        }
        self.instances = self.client.update_instances_number(
            self.function,
            **attrs
        )
        assert isinstance(self.function, function.Function)

        self.addCleanup(
            self.client.delete_function,
            self.function
        )

    def test_instances_config(self):
        inst = list(self.client.reserved_instances_config())
        self.assertEqual(self.function.func_urn, inst[0].function_urn)
        self.assertEqual(False, inst[0].idle_mode)
        self.assertEqual(False, inst[0].idle_mode)
        self.assertEqual(2, inst[0].min_count)
        self.assertEqual("version", inst[0].qualifier_type)
        self.assertEqual("latest", inst[0].qualifier_name)

    def test_instances(self):
        inst = list(self.client.reserved_instances())
        self.assertEqual(self.function.func_urn, inst[0].func_urn)
        self.assertEqual(2, inst[0].count)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid
import openstack

from otcextensions.tests.functional.sdk.apig import TestApiG
from otcextensions.tests.ssl import SelfSignedCertificateGenerator

_logger = openstack._log.setup_logging('openstack')


class TestDomain(TestApiG):

    def setUp(self):
        super(TestDomain, self).setUp()
        self.network_client = self.conn.dns
        self.suffix = uuid.uuid4().hex[:4]
        self.zone_name = self.suffix + 'dns.sdk-apig-zone-public.com.'
        self.create_gateway()
        self.gateway_id = TestDomain.gateway.id
        # self.gateway_id = "be76ca6de5fe4aa7af503c03b3b44dea"

        group_attrs = {
            "name": f"api_group_{self.suffix}",
            "remark": f"API group {self.suffix}"
        }
        self.group = self.client.create_api_group(
            gateway=self.gateway_id,
            **group_attrs
        )
        self.assertIsNotNone(self.group.id)

        self.zone = self.network_client.create_zone(
            name=self.zone_name
        )
        self.network_client.wait_for_zone(self.zone)

        attrs = {
            "url_domain": self.zone.name
        }
        self.domain = self.client.bind_domain_name(
            gateway=self.gateway_id,
            group=self.group.id,
            **attrs
        )
        self.assertIsNotNone(self.group.id)

        generator = SelfSignedCertificateGenerator(self.zone_name)
        generator.generate_private_key()
        generator.generate_certificate()
        attrs = {
            "name": f"cert_demo_{self.suffix}",
            "private_key": generator.get_private_key(),
            "cert_content": generator.get_certificate()
        }
        self.bind = self.client.create_certificate_for_domain_name(
            gateway=self.gateway_id,
            group=self.group.id,
            domain=self.domain.id,
            **attrs
        )
        self.assertEqual(self.bind.name, attrs["name"])

        self.addCleanup(self.delete_gateway())
        self.addCleanup(
            self.client.delete_api_group,
            gateway=self.gateway_id,
            api_group=self.group.id,
        )
        self.addCleanup(
            self.client.delete_certificate,
            certificate=self.bind.ssl_id,
        )
        self.addCleanup(
            self.client.unbind_domain_name,
            gateway=self.gateway_id,
            group=self.group.id,
            domain=self.domain.id,
        )

    def tearDown(self):
        if self.zone:
            try:
                self.network_client.delete_zone(self.zone)
                self.network_client.wait_for_delete_zone(self.zone)
            except openstack.exceptions.SDKException as e:
                _logger.warning('Got exception during clearing resources %s'
                                % e.message)
        super(TestDomain, self).tearDown()

    def test_update_domain_name_bound(self):
        attrs = {
            "min_ssl_version": "TLSv1.2"
        }
        updated = self.client.update_domain_name_bound(
            gateway=self.gateway_id,
            group=self.group.id,
            domain=self.domain.id,
            **attrs
        )
        self.assertEqual(updated.min_ssl_version, attrs["min_ssl_version"])

    # def test_unbind_certificate_from_domain(self):
    #     unbind = self.client.unbind_certificate_from_domain_name(
    #         gateway=self.gateway_id,
    #         group=self.group.id,
    #         domain=self.domain.id,
    #         certificate=self.bind.id,
    #     )
    #     self.assertIsNotNone(unbind)

    def test_disable_debug(self):
        debug = self.client.enable_debug_domain_name(
            gateway=self.gateway_id,
            group=self.group.id,
            domain=self.domain.id,
            enable=False,
        )
        self.assertEqual(
            debug.sl_domain_access_enabled,
            False
        )

    def test_get_bound_certificate(self):
        cert = self.client.get_bound_certificate(
            gateway=self.gateway_id,
            group=self.group.id,
            domain=self.domain.id,
            certificate=self.bind.ssl_id,
        )
        self.assertEqual(
            cert.id,
            self.bind.ssl_id
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource

from otcextensions.sdk import sdk_resource
from otcextensions.sdk.cce.v1 import _base


class VolumeSpec(sdk_resource.Resource):
    # Properties
    #: Disk Type (root, data)
    disk_type = resource.Body('diskType')
    #: Disk Size (40G for root, 100G-32768G for data)
    disk_size = resource.Body('diskSize', type=int)
    #: Volume type (SATA, SAS, SSD)
    volume_type = resource.Body('volumeType')


class CapacitySpec(sdk_resource.Resource):
    # Properties
    #: CPU
    #: *Type: str*
    cpu = resource.Body('cpu')
    #: Memory
    #: *Type: str*
    memory = resource.Body('memory')
    #: Pods
    #: *Type: str*
    pods = resource.Body('pods')


class StatusSpec(sdk_resource.Resource):
    # Properties
    #: Capacity - maximal resources capacity
    #: *Type: CapacitySpec*
    capacity = resource.Body('capacity', type=CapacitySpec)
    #: Allocatable - available resources capacity
    #: *Type: CapacitySpec*
    allocatable = resource.Body('allocatable', type=CapacitySpec)
    #: Conditions - health conditions
    conditions = resource.Body('conditions', type=list, list_type=dict)


class NodeSpec(sdk_resource.Resource):
    # Properties
    #: Cluster UUID
    cluster_uuid = resource.Body('clusteruuid')
    #: Private IP
    private_ip = resource.Body('privateip')
    #: Public IP
    public_ip = resource.Body('publicip')
    #: Flavor (mandatory)
    flavor = resource.Body('flavor')
    #: Label
    label = resource.Body('label')
    #: CPU
    cpu = resource.Body('cpu')
    #: Memory
    memory = resource.Body('memory')
    #: availability zone
    availability_zone = resource.Body('az')
    #: volume (mandatory)
    volume = resource.Body('volume', type=list, list_type=VolumeSpec)
    #: SSH Key (mandatory)
    ssh_key = resource.Body('sshkey')
    #: status
    status = resource.Body('status', type=StatusSpec)
    #: Tags (array in format key.value)
    tags = resource.Body('tags', type=list)
    #: assign_floating_ip - whether to assign floating IP to the server
    #: (used only during create)
    #: *Type:bool*
    assign_floating_ip = resource.Body('snat', type=bool)
    #: tags (only used in creation). Format: "KEY.VALUE"
    #: *Type:list*
    tags = resource.Body('tags', type=list)


class ClusterNode(_base.Resource):
    base_path = '/clusters/%(cluster_uuid)s/hosts'
    allow_create = True
    allow_delete = True
    allow_list = True
    allow_get = True
    # Responses do not have body
    # has_body = False

    # Properties
    #: Spec
    spec = resource.Body('spec', type=NodeSpec)
    #: Status
    status = resource.Body('status')
    #: Replicas count
    replica_count = resource.Body('replicas', type=int)
    #: Message
    message = resource.Body('message')

    cluster_uuid = resource.URI('cluster_uuid')

    @classmethod
    def new(cls, **kwargs):
        if 'kind' not in kwargs:
            kwargs['kind'] = 'host'
        if 'apiVersion' not in kwargs:
            kwargs['apiVersion'] = 'v1'
        return cls(_synchronized=False, **kwargs)

    def __getattribute__(self, name):
        """Return an attribute on this instance

        This is mostly a pass-through except for a specialization on
        the 'id' name, as this can exist under a different name via the
        `alternate_id` argument to resource.Body.
        """
        if name == 'id' or name == 'name':
            if name in self._body:
                return self._body[name]
            else:
                try:
                    metadata = self._body['metadata']
                    if name == 'id':
                        if isinstance(metadata, dict):
                            return metadata['uuid']
                        elif isinstance(metadata, _base.Metadata):
                            return metadata._body[metadata._alternate_id()]
                    else:
                        if isinstance(metadata, dict):
                            return metadata['name']
                        elif isinstance(metadata, _base.Metadata):
                            return metadata.name
                except KeyError:
                    return None
        else:
            return object.__getattribute__(self, name)

    @classmethod
    def list(cls, session,
             endpoint_override=None, headers=None, **params):
        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")

        session = cls._get_session(session)

        # cls._query_mapping._validate(params, base_path=cls.base_path)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = cls.base_path % params

        # Build additional arguments to the GET call
        get_args = cls._prepare_override_args(
            endpoint_override=endpoint_override,
            additional_headers=headers)

        while uri:
            response = session.get(
                uri,
                params=query_params.copy(),
                **get_args
            )
            exceptions.raise_from_response(response)
            data = response.json()
            spec = data.get('spec', None)
            resources = []
            if spec:
                resources = spec.get('hostList', [])

            for raw_resource in resources:
                # Do not allow keys called "self" through. Glance chose
                # to name a key "self", so we need to pop it out because
                # we can't send it through cls.existing and into the
                # Resource initializer. "self" is already the first
                # argument and is practically a reserved word.
                raw_resource.pop("self", None)

                if cls.resource_key and cls.resource_key in raw_resource:
                    raw_resource = raw_resource[cls.resource_key]

                value = cls.existing(**raw_resource)

                yield value
            return

    def delete(self, session, error_message=None,
               endpoint_override=None, headers=None, params=None):
        """Delete the remote resource based on this instance.

        This function overrides default Resource.delete to enable headers

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`

        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_update` is not set to ``True``.
        """
        if not self.allow_delete:
            raise exceptions.MethodNotSupported(self, "delete")

        request = self._prepare_request(requires_id=False)
        session = self._get_session(session)

        # Build additional arguments to the DELETE call
        delete_args = self._prepare_override_args(
            endpoint_override=endpoint_override,
            request_headers=request.headers,
            additional_headers=headers)
        if params:
            delete_args['params'] = params

        body = {
            'hosts': [
                {'name': self.name}
            ]
        }

        response = session.delete(request.url,
                                  json=body,
                                  **delete_args)
        kwargs = {}
        if error_message:
            kwargs['error_message'] = error_message

        self._translate_response(response, has_body=False, **kwargs)
        return self

    def create(self, session, prepend_key=True, requires_id=True,
               endpoint_override=None, headers=None):
        if not self.allow_create:
            raise exceptions.MethodNotSupported(self, "create")

        session = self._get_session(session)

        if self.create_method == 'PUT':
            request = self._prepare_request(requires_id=True,
                                            prepend_key=prepend_key)
            req_args = self._prepare_override_args(
                endpoint_override=endpoint_override,
                request_headers=request.headers,
                additional_headers=headers)
            response = session.put(request.url,
                                   json=request.body, **req_args)
        elif self.create_method == 'POST':
            request = self._prepare_request(requires_id=False,
                                            prepend_key=prepend_key)
            req_args = self._prepare_override_args(
                endpoint_override=endpoint_override,
                request_headers=request.headers,
                additional_headers=headers)
            response = session.post(request.url,
                                    json=request.body, **req_args)
        else:
            raise exceptions.ResourceFailure(
                msg="Invalid create method: %s" % self.create_method)

        # This is an only difference to the existing sdk_resource.create
        self._translate_response(response, has_body=False)

        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from openstack import _log
from openstack import resource

from otcextensions.sdk.vpc.v1 import vpc
from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestService(base.BaseFunctionalTest):
    _vpc: vpc.Vpc
    _seed: str

    @property
    def seed(self):
        if not hasattr(self, '_seed') or self._seed is None:
            self._seed = uuid.uuid4().hex[:8]
        return self._seed

    def setUp(self):
        super().setUp()

        attrs = {
            'name': 'test-vpc-' + self.seed,
            'cidr': '192.168.0.0/16'
        }
        self.vpc = self.conn.vpc.create_vpc(**attrs)
        self.addCleanup(self.conn.vpc.delete_vpc, self.vpc)

    def test_create_subnet(self):
        cidr = self.vpc.cidr
        gateway, _ = cidr.split("/")
        gateway = gateway[:-2] + ".1"  # .0 -> .1

        attrs = {
            'vpc_id': self.vpc.id,
            'name': 'test-subnet-' + self.seed,
            'cidr': cidr,
            'gateway_ip': gateway,
            'dns_list': [
                "100.125.4.25",
                "100.125.129.199",
            ],
        }
        subnet = self.conn.vpc.create_subnet(**attrs)
        self.assertIsNotNone(subnet.id)
        self.addCleanup(self._delete_subnet, subnet)

        self.assertIsNotNone(subnet.status)
        self.assertIsNotNone(subnet.neutron_subnet_id)
        self.assertIsNotNone(subnet.neutron_network_id)

    def test_get_subnet(self):
        subnet = self._create_subnet()
        found = self.conn.vpc.get_subnet(subnet.id)

        self.assertEqual(found, subnet)

    def test_find_subnet(self):
        subnet = self._create_subnet()
        found = self.conn.vpc.find_subnet(subnet.name)

        self.assertEqual(found, subnet)

    def test_update_subnet(self):
        subnet = self._create_subnet()

        new_attrs = {
            'name': 'test-updated-' + self.seed,
            'dns_list': [
                "100.125.4.25",
                "8.8.8.8",
            ],
        }
        updated = self.conn.vpc.update_subnet(subnet, **new_attrs)
        self.assertEqual(updated.name, new_attrs['name'])
        self.assertEqual(updated.dns_list, new_attrs['dns_list'])

    def test_delete_subnet(self):
        subnet = self._create_subnet(False)
        self.conn.vpc.delete_subnet(subnet, ignore_missing=False)
        resource.wait_for_delete(self.conn.vpc, subnet, 2, 120)

    def test_list_subnets(self):
        subnet = self._create_subnet()

        subnets = list(self.conn.vpc.subnets(vpc_id=subnet.vpc_id))
        self.assertGreaterEqual(len(subnets), 1)

        self.assertIn(subnet, subnets)

    def _create_subnet(self, remove=True):
        cidr: str = self.vpc.cidr
        gateway, _ = cidr.split("/")
        gateway = gateway[:-2] + ".1"  # .0 -> .1

        attrs = {
            'vpc_id': self.vpc.id,
            'name': 'test-subnet-' + self.seed,
            'cidr': cidr,
            'gateway_ip': gateway,
            'dns_list': [
                "100.125.4.25",
                "100.125.129.199",
            ],
        }
        subnet = self.conn.vpc.create_subnet(**attrs)
        resource.wait_for_status(self.conn.vpc, subnet, "ACTIVE", None, 2, 20)

        if remove:
            self.addCleanup(self._delete_subnet, subnet)
        return subnet

    def _delete_subnet(self, subnet):
        resource.wait_for_status(self.conn.vpc, subnet, "ACTIVE", None, 2, 20)
        self.conn.vpc.delete_subnet(subnet, ignore_missing=False)
        resource.wait_for_delete(self.conn.vpc, subnet, 2, 60)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from otcextensions.sdk.dws.v1 import cluster as _cluster


class Snapshot(resource.Resource):
    base_path = '/snapshots'

    resource_key = 'snapshot'
    resources_key = 'snapshots'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True

    # Properties
    #: ID of the cluster for which snapshots are created.
    cluster_id = resource.Body('cluster_id')
    #: Time when a snapshot starts to be created.
    created_at = resource.Body('started')
    #: Snapshot description.
    description = resource.Body('description')
    #: Snapshot size, in GB.
    size = resource.Body('size', type=float)
    #: Snapshot status:
    #:  - CREATING
    #:  - AVAILABLE
    #:  - UNAVAILABLE
    status = resource.Body('status')
    #: Snapshot type. It can be:
    #:  - MANUAL
    #:  - AUTOMATED
    type = resource.Body('type')
    #: Time when a snapshot is complete.
    updated_at = resource.Body('finished')


class Restore(_cluster.Cluster):
    base_path = '/snapshots/%(snapshot_id)s/actions'

    resources_key = None
    resource_key = 'restore'

    snapshot_id = resource.URI('snapshot_id')

    # capabilities
    allow_create = True
    allow_fetch = False
    allow_delete = False
    allow_list = False
    allow_commit = False
    allow_patch = False

    #: Returns `~otcextensions.sdk.dws.v1.cluster.Cluster` object.
    cluster = resource.Body('cluster', type=_cluster.Cluster)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.sdk.dns.v2 import nameserver


FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
EXAMPLE = {
    "priority": 1,
    "address": "fake.ip",
    "hostname": "fake.hostname",
}


class TestNS(base.TestCase):

    def test_basic(self):
        sot = nameserver.NameServer()

        self.assertEqual('/zones/%(zone_id)s/nameservers', sot.base_path)

        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_update)

    def test_make_it(self):

        sot = nameserver.NameServer(**EXAMPLE)
        self.assertEqual(EXAMPLE['priority'], sot.priority)
        self.assertEqual(EXAMPLE['address'], sot.address)
        self.assertEqual(EXAMPLE['hostname'], sot.hostname)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''DCS Config params v1 action implementations'''
import logging
import argparse

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListInstanceParam(command.Lister):
    _description = _('List configurational parameters of a single DCS '
                     'instance')
    columns = ('id', 'name', 'value', 'default_value')

    def get_parser(self, prog_name):
        parser = super(ListInstanceParam, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('Name or ID of the instance')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcs

        inst = client.find_instance(name_or_id=parsed_args.instance,
                                    ignore_missing=False)
        data = client.instance_params(
            instance={'id': inst.id},
        )

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns,
                 ) for s in data))
        return table


class UpdateInstanceParam(command.Command):
    _description = _('Update instance configurational parameters')

    def get_parser(self, prog_name):
        parser = super(UpdateInstanceParam, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('Name or ID of the DCS instance to take backup from.')
        )
        parser.add_argument(
            '--param',
            metavar='<id:name:value>',
            required=True,
            action='append',
            help=_('Parameter pair in format ID:NAME:VALUE.')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dcs

        params = []

        for param in parsed_args.param:
            param_parts = param.split(':')
            if 3 == len(param_parts):
                param_struct = {
                    'param_id': param_parts[0],
                    'param_name': param_parts[1],
                    'param_value': param_parts[2]
                }
                params.append(param_struct)
            else:
                msg = _('Cannot parse tag information')
                raise argparse.ArgumentTypeError(msg)

        if params:
            inst = client.find_instance(name_or_id=parsed_args.instance,
                                        ignore_missing=False)

            return client.update_instance_params(
                instance={'id': inst.id}, params=params
            )


class ShowInstanceParam(command.ShowOne):
    _description = _('Show the details of a single instance parameter')

    def get_parser(self, prog_name):
        parser = super(ShowInstanceParam, self).get_parser(prog_name)

        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('Name or UUID of the instance.')
        )
        parser.add_argument(
            '--param',
            metavar='<param>',
            required=True,
            help=_('ID or name of the parameter.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dcs

        inst = client.find_instance(name_or_id=parsed_args.instance,
                                    ignore_missing=False)
        data = client.instance_params(
            instance={'id': inst.id},
        )

        criteria = parsed_args.param

        found = None

        for param in data:
            if param.id == criteria or criteria in param.name:
                found = param
                break

        if found:
            display_columns, columns = _get_columns(found)
            data = utils.get_item_properties(found, columns)

            return (display_columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
List resources from the Key Manager service.
"""


def list_secrets(conn):
    print("List Secrets:")

    for secret in conn.key_manager.secrets():
        print(secret)


def list_secrets_query(conn):
    print("List Secrets:")

    for secret in conn.key_manager.secrets(
            secret_type="symmetric",
            expiration="gte:2020-01-01T00:00:00"):
        print(secret)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack.tests.unit import base
from otcextensions.sdk.modelartsv2.v2 import dataset
from otcextensions.tests.unit.sdk.modelartsv2.v2 import examples
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = examples.DATASET

EXAMPLE_CREATE = {
    "workspace_id": "0",
    "dataset_name": "dataset-de83",
    "dataset_type": 400,
    "data_sources": [
        {
            "data_type": 0,
            "data_path": "/test-obs/table/input/",
            "with_column_header": True,
        }
    ],
    "description": "",
    "work_path": "/test-obs/table/output/",
    "work_path_type": 0,
    "schema": [
        {"schema_id": 1, "name": "150", "type": "STRING"},
        {"schema_id": 2, "name": "4", "type": "STRING"},
        {"schema_id": 3, "name": "setosa", "type": "STRING"},
        {"schema_id": 4, "name": "versicolor", "type": "STRING"},
        {"schema_id": 5, "name": "virginica", "type": "STRING"},
    ],
    "import_data": True,
}


class TestDataset(base.TestCase):
    def setUp(self):
        super(TestDataset, self).setUp()

    def test_basic(self):
        sot = dataset.Dataset()

        self.assertEqual("/datasets", sot.base_path)
        self.assertEqual("datasets", sot.resources_key)
        self.assertEqual(None, sot.resource_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

        self.assertDictEqual(
            {
                "check_running_task": "check_running_task",
                "contain_versions": "contain_versions",
                "dataset_type": "dataset_type",
                "file_preview": "file_preview",
                "limit": "limit",
                "marker": "marker",
                "offset": "offset",
                "order": "order",
                "running_task_type": "running_task_type",
                "search_content": "search_content",
                "sort_by": "sort_by",
                "support_export": "support_export",
                "train_evaluate_ratio": "train_evaluate_ratio",
                "version_format": "version_format",
                "with_labels": "with_labels",
                "workspace_id": "workspace_id",
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = dataset.Dataset(**EXAMPLE)
        self.assertEqual(sot.id, EXAMPLE["dataset_id"])
        for key, value in EXAMPLE.items():
            if key == "labels":
                for ix, data in enumerate(value):
                    self.assertEqual(sot.labels[ix].name, data["name"])
                    self.assertEqual(sot.labels[ix].type, data["type"])
                    self.assertEqual(
                        sot.labels[ix].attributes, data["attributes"]
                    )
                    self.assertEqual(
                        sot.labels[ix].property.color,
                        data["property"]["@modelarts:color"],
                    )
                    self.assertEqual(
                        sot.labels[ix].property.shortcut,
                        data["property"]["@modelarts:shortcut"],
                    )
            else:
                assert_attributes_equal(self, getattr(sot, key), value)

    def test_create_sot(self):
        sot = dataset.Dataset(**EXAMPLE_CREATE)
        assert_attributes_equal(self, sot, EXAMPLE_CREATE)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource

from otcextensions.sdk.dds.v3.instance import DatastoreSpec


class RecycleInstanceSpec(resource.Resource):
    id = resource.Body('id')
    name = resource.Body('name')
    mode = resource.Body('mode')
    datastore = resource.Body('datastore', type=DatastoreSpec)
    pay_mode = resource.Body('pay_mode')
    enterprise_project_id = resource.Body('enterprise_project_id')
    backup_id = resource.Body('backup_id')
    created_at = resource.Body('created_at')
    deleted_at = resource.Body('deleted_at')
    retained_until = resource.Body('retained_until')
    status = resource.Body('status')


class RecycleInstance(resource.Resource):
    base_path = '/recycle-instances'

    allow_list = True
    requires_id = False
    total_count = resource.Body('total_count', type=int)
    instances = resource.Body('instances', type=list,
                              list_type=RecycleInstanceSpec)

    @classmethod
    def list(cls, session, paginated=True, base_path=None, **params):
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)
        response = session.get(
            base_path,
            headers={"Content-Type": "application/json"},
            params={},
            microversion=microversion,
        )
        exceptions.raise_from_response(response)
        data = response.json()
        resources = data
        if not isinstance(resources, list):
            resources = [resources]

        for raw_resource in resources:
            raw_resource.pop('self', None)
            value = cls.existing(
                microversion=microversion,
                connection=session._get_connection(),
                **raw_resource,)
            yield value
        else:
            return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.identity.v3 import agency


FAKE_ID = "945d-fe449be00148"
EXAMPLE = {
    "trust_domain_name": "exampledomain",
    "description": " testsfdas ",
    "trust_domain_id": "b3f266d0c08544a0859740de8b84e850",
    "id": "afca8ddf2e92469a8fd26a635da5206f",
    "duration": None,
    "create_time": "2017-01-04T09:09:15.000000",
    "expire_time": None,
    "domain_id": "0ae9c6993a2e47bb8c4c7a9bb8278d61",
    "name": "exampleagency"
}


class TestAgency(base.TestCase):

    def setUp(self):
        super(TestAgency, self).setUp()

    def test_basic(self):
        sot = agency.Agency()

        self.assertEqual('/v3.0/OS-AGENCY/agencies', sot.base_path)
        self.assertEqual('agencies', sot.resources_key)
        self.assertEqual('agency', sot.resource_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

        self.assertDictEqual({
            'domain_id': 'domain_id',
            'limit': 'limit',
            'marker': 'marker',
            'name': 'name',
            'trust_domain_id': 'trust_domain_id'},
            sot._query_mapping._mapping
        )

    def test_make_it(self):

        sot = agency.Agency(connection=self.cloud, **EXAMPLE)
        # Check how the override with "real" connection works
        self.assertEqual(
            'https://identity.example.com/v3.0/OS-AGENCY/agencies',
            sot.base_path)

        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['trust_domain_id'], sot.trust_domain_id)
        self.assertEqual(EXAMPLE['create_time'], sot.created_at)
        self.assertEqual(EXAMPLE['expire_time'], sot.expire_at)
        self.assertEqual(EXAMPLE['domain_id'], sot.domain_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

from unittest import mock

from openstack.tests.unit import base

from otcextensions.sdk.dms.v1 import instance


JSON_DATA = {
    "name": "kafka-test",
    "description": "",
    "engine": "kafka",
    "engine_version": "2.3.0",
    "storage_space": 600,
    "access_user": "",
    "password": "",
    "vpc_id": "1e93f86e-13af-46c8-97d6-d40fa62b76c2",
    "security_group_id": "0aaa0033-bf7f-4c41-a6c2-18cd04cad2c8",
    "subnet_id": "b5fa806c-35e7-4299-b659-b39398dd4718",
    "available_zones": ["d573142f24894ef3bd3664de068b44b0"],
    "product_id": "00300-30308-0--0",
    "maintain_begin": "22:00",
    "maintain_end": "02:00",
    "ssl_enable": False,
    "enable_publicip": False,
    "publicip_id": "",
    "specification": "100MB",
    "partition_num": "300",
    "retention_policy": "produce_reject",
    "connector_enable": False,
    "storage_spec_code": "dms.physical.storage.ultra"
}


class TestInstance(base.TestCase):

    def setUp(self):
        super(TestInstance, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock()

    def test_basic(self):
        sot = instance.Instance()

        self.assertEqual('/instances', sot.base_path)
        self.assertEqual('instances', sot.resources_key)

        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

        self.assertDictEqual({
            'engine_name': 'engine',
            'exact_match_name': 'exactMatchName',
            'include_failure': 'includeFailure',
            'limit': 'limit',
            'marker': 'marker',
            'name': 'name',
            'status': 'status'},
            sot._query_mapping._mapping
        )

    def test_make_it(self):
        sot = instance.Instance(**JSON_DATA)
        self.assertEqual(JSON_DATA['name'], sot.name)
        self.assertEqual(JSON_DATA['access_user'], sot.access_user)
        self.assertEqual(JSON_DATA['available_zones'], sot.availability_zones)
        self.assertEqual(JSON_DATA.get('charging_mode', None),
                         sot.charging_mode)
        self.assertEqual(JSON_DATA.get('connect_address', None),
                         sot.connect_address)
        self.assertEqual(JSON_DATA.get('created_at', None),
                         sot.created_at)
        self.assertEqual(JSON_DATA.get('description', None),
                         sot.description)
        self.assertEqual(JSON_DATA.get('engine', None),
                         sot.engine_name)
        self.assertEqual(JSON_DATA.get('engine_version', None),
                         sot.engine_version)
        self.assertEqual(JSON_DATA.get('instance_id', None),
                         sot.id)
        self.assertEqual(JSON_DATA.get('enable_publicip', False),
                         sot.is_public)
        self.assertEqual(JSON_DATA.get('ssl_enable', None),
                         sot.is_ssl)
        self.assertEqual(JSON_DATA.get('kafka_public_status', None),
                         sot.kafka_public_status)
        self.assertEqual(JSON_DATA.get('maintain_end', None),
                         sot.maintenance_end)
        self.assertEqual(JSON_DATA.get('maintain_begin', None),
                         sot.maintenance_start)
        self.assertEqual(int(JSON_DATA.get('partition_num', None)),
                         sot.max_partitions)
        self.assertEqual(JSON_DATA.get('password', None),
                         sot.password)
        self.assertEqual(JSON_DATA.get('port', None),
                         sot.port)
        self.assertEqual(JSON_DATA.get('product_id', None),
                         sot.product_id)
        self.assertEqual(JSON_DATA.get('public_bandwidth', None),
                         sot.public_bandwidth)
        self.assertEqual(JSON_DATA.get('retention_policy', None),
                         sot.retention_policy)
        self.assertEqual(JSON_DATA.get('vpc_id', None),
                         sot.router_id)
        self.assertEqual(JSON_DATA.get('vpc_name', None),
                         sot.router_name)
        self.assertEqual(JSON_DATA.get('security_group_id', None),
                         sot.security_group_id)
        self.assertEqual(JSON_DATA.get('security_group_name', None),
                         sot.security_group_name)
        self.assertEqual(JSON_DATA.get('service_type', None),
                         sot.service_type)
        self.assertEqual(JSON_DATA.get('specification', None),
                         sot.spec)
        self.assertEqual(JSON_DATA.get('resource_spec_code', None),
                         sot.spec_code)
        self.assertEqual(JSON_DATA.get('status', None),
                         sot.status)
        self.assertEqual(JSON_DATA.get('storage_resource_id', None),
                         sot.storage_resource_id)
        self.assertEqual(JSON_DATA.get('storage_spec_code', None),
                         sot.storage_spec_code)
        self.assertEqual(JSON_DATA.get('storage_type', None),
                         sot.storage_type)
        self.assertEqual(JSON_DATA.get('storage_space', None),
                         sot.storage)
        self.assertEqual(JSON_DATA.get('subnet_id', None),
                         sot.network_id)
        self.assertEqual(JSON_DATA.get('total_storage_space', None),
                         sot.total_storage)
        self.assertEqual(JSON_DATA.get('type', None),
                         sot.type)
        self.assertEqual(JSON_DATA.get('used_storage_space', None),
                         sot.storage_type)
        self.assertEqual(JSON_DATA.get('user_id', None),
                         sot.user_id)
        self.assertEqual(JSON_DATA.get('user_name', None),
                         sot.user_name)

    def test_restart(self):
        sot = instance.Instance(id='1')
        response = mock.Mock()
        response.status_code = 200

        self.sess.post.return_value = response

        sot.restart(self.sess)

        self.sess.post.assert_called_with(
            '/instances/action',
            json={'action': 'restart', 'instances': ['1']}
        )

        sot.restart_batch(self.sess, ['1', '2'])

        self.sess.post.assert_called_with(
            '/instances/action',
            json={'action': 'restart', 'instances': ['1', '2']}
        )

    def test_delete(self):
        sot = instance.Instance()
        response = mock.Mock()
        response.status_code = 200

        self.sess.post.return_value = response

        sot.delete_batch(self.sess, ['1', '2'])

        self.sess.post.assert_called_with(
            '/instances/action',
            json={'action': 'delete', 'instances': ['1', '2']}
        )

        sot.delete_failed(self.sess)

        self.sess.post.assert_called_with(
            '/instances/action',
            json={'action': 'delete', 'allFailure': 'kafka'}
        )



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
List all CSS Clusters
"""

import openstack


openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

print(conn.css.get_certificate())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from otcextensions.osclient.mrs.v1 import cluster
from otcextensions.tests.unit.osclient.mrs.v1 import fakes


class TestListClusterHost(fakes.TestMrs):
    objects = fakes.FakeHost.create_multiple(3)

    columns = (
        'id', 'name', 'status', 'flavor', 'type',
        'ip', 'mem', 'cpu', 'data_volume_size'
    )

    data = []

    for s in objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListClusterHost, self).setUp()

        self.cmd = cluster.ListClusterHost(self.app, None)

        self.client.hosts = mock.Mock()
        self.client.api_mock = self.client.hosts

    def test_default(self):
        arglist = [
            'cluster_id'
        ]
        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            cluster_id='cluster_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import json

from openstack import exceptions
from openstack import resource


class ParameterSpec(resource.Resource):
    #: Parameter name.
    label = resource.Body("label")
    #: Whether a parameter is mandatory.
    required = resource.Body("required", type=bool)
    #: Parameter value.
    value = resource.Body("value")


class BuiltInModel(resource.Resource):
    base_path = "/built-in-algorithms"

    resources_key = "models"

    # capabilities
    allow_list = True
    allow_fetch = True

    _query_mapping = resource.QueryParameters(
        "limit",
        "order",
        "offset",
        "search_content",
        "sort_by",
        limit="per_page",
        offset="page",
        sort_by="sortBy",
    )

    # Properties
    #: Time when a model is created.
    created_at = resource.Body("create_time", type=int)
    #: Engine ID of a model.
    engine_id = resource.Body("engine_id", type=int)
    #: Engine name of a model.
    engine_name = resource.Body("engine_name")
    #: Engine version of a model.
    engine_version = resource.Body("engine_version")
    #: Whether the request is successful.
    is_success = resource.Body("is_success", type=bool)
    #: Dataset format required by a model.
    model_dataset_format = resource.Body("model_dataset_format")
    #: URL of the model description.
    model_description_url = resource.Body("model_description_url")
    #: Model ID.
    model_id = resource.Body("model_id")
    #: Model name.
    model_name = resource.Body("model_name")
    #: Model precision.
    model_precision = resource.Body("model_precision")
    #: Model size, in bytes.
    model_size = resource.Body("model_size", type=int)
    #: Model training dataset.
    model_train_dataset = resource.Body("model_train_dataset")
    #: Model usage.
    model_usage = resource.Body("model_usage", type=int)
    #: Model name.
    # name = resource.Body("name", alias="model_name")
    #: Running parameters of a model.
    parameter = resource.Body("parameter", type=list, list_type=ParameterSpec)

    @classmethod
    def existing(cls, connection=None, **kwargs):
        """Create an instance of an existing remote resource.

        When creating the instance set the ``_synchronized`` parameter
        of :class:`Resource` to ``True`` to indicate that it represents the
        state of an existing server-side resource. As such, all attributes
        passed in ``**kwargs`` are considered "clean", such that an immediate
        :meth:`update` call would not generate a body of attributes to be
        modified on the server.

        :param dict kwargs: Each of the named arguments will be set as
            attributes on the resulting Resource object.
        """
        parameter = kwargs.get("parameter")
        if parameter and isinstance(parameter, str):
            kwargs["parameter"] = json.loads(parameter)
        return cls(_synchronized=True, connection=connection, **kwargs)

    @classmethod
    def _get_one_match(cls, name_or_id, results):
        """Given a list of results, return the match"""
        the_result = None
        for maybe_result in results:
            id_value = str(maybe_result.model_id)
            name_value = maybe_result.model_name

            if (id_value == name_or_id) or (name_value == name_or_id):
                # Only allow one resource to be found. If we already
                # found a match, raise an exception to show it.
                if the_result is None:
                    the_result = maybe_result
                else:
                    msg = "More than one %s exists with the name '%s'."
                    msg = msg % (cls.__name__, name_or_id)
                    raise exceptions.DuplicateResource(msg)

        return the_result

    @classmethod
    def find(
        cls,
        session,
        name_or_id: str,
        ignore_missing: bool = True,
        **params,
    ):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param name_or_id: This resource's identifier, if needed by
            the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the resource does not exist.  When set to ``True``, None will be
            returned when attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
            underlying methods, such as to
            :meth:`~openstack.resource.Resource.existing` in order to pass on
            URI parameters.

        :return: The :class:`Resource` object matching the given name or id
            or None if nothing matches.
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
            than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
            is found and ignore_missing is ``False``.
        """
        session = cls._get_session(session)

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None

        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id)
        )
